@@ -2,6 +2,7 @@ import os
 from typing import Dict, List, Tuple
 
 import torch
+import torch.distributed as dist
 from coqpit import Coqpit
 from torch import nn
 from torch.utils.data import DataLoader
@@ -164,7 +165,14 @@ class BaseTTS(BaseModel):
         }
 
     def get_data_loader(
-        self, config: Coqpit, ap: AudioProcessor, is_eval: bool, data_items: List, verbose: bool, num_gpus: int
+        self,
+        config: Coqpit,
+        ap: AudioProcessor,
+        is_eval: bool,
+        data_items: List,
+        verbose: bool,
+        num_gpus: int,
+        rank: int = None,
     ) -> "DataLoader":
         if is_eval and not config.run_eval:
             loader = None
@@ -186,7 +194,7 @@ class BaseTTS(BaseModel):
             if hasattr(self, "make_symbols"):
                 custom_symbols = self.make_symbols(self.config)
 
-            # init dataloader
+            # init dataset
             dataset = TTSDataset(
                 outputs_per_step=config.r if "r" in config else 1,
                 text_cleaner=config.text_cleaner,
@@ -212,13 +220,15 @@ class BaseTTS(BaseModel):
                 else None,
             )
 
-            if config.use_phonemes and config.compute_input_seq_cache:
+            # pre-compute phonemes
+            if config.use_phonemes and config.compute_input_seq_cache and rank in [None, 0]:
                 if hasattr(self, "eval_data_items") and is_eval:
                     dataset.items = self.eval_data_items
                 elif hasattr(self, "train_data_items") and not is_eval:
                     dataset.items = self.train_data_items
                 else:
-                    # precompute phonemes to have a better estimate of sequence lengths.
+                    # precompute phonemes for precise estimate of sequence lengths.
+                    # otherwise `dataset.sort_items()` uses raw text lengths
                     dataset.compute_input_seq(config.num_loader_workers)
 
                     # TODO: find a more efficient solution
@@ -228,9 +238,17 @@ class BaseTTS(BaseModel):
                     else:
                         self.train_data_items = dataset.items
 
-            dataset.sort_items()
+            # halt DDP processes for the main process to finish computing the phoneme cache
+            if num_gpus > 1:
+                dist.barrier()
 
+            # sort input sequences from short to long
+            dataset.sort_and_filter_items(config.get("sort_by_audio_len", default=False))
+
+            # sampler for DDP
             sampler = DistributedSampler(dataset) if num_gpus > 1 else None
+
+            # init dataloader
             loader = DataLoader(
                 dataset,
                 batch_size=config.eval_batch_size if is_eval else config.batch_size,
